@@ -9,11 +9,12 @@ use warnings qw(all);
 use AnyEvent;
 use Any::Moose;
 use Any::Moose qw(::Util::TypeConstraints);
+use Carp qw(confess);
 use Net::Curl::Share;
 
 use AnyEvent::Net::Curl::Queued::Multi;
 
-our $VERSION = '0.035'; # VERSION
+our $VERSION = '0.036'; # VERSION
 
 
 has allow_dups  => (is => 'ro', isa => 'Bool', default => 0);
@@ -59,26 +60,13 @@ sub dequeue         { shift @{shift->queue} }
 sub count           { scalar @{shift->queue} }
 
 
-our %share; # PLEASE, MAKE IT STOP!!! IT HURTS MY BRAIN!!!
 has share       => (
     is      => 'ro',
     isa     => 'Net::Curl::Share',
-    default => sub {
-        my $share = Net::Curl::Share->new;
-        $share{$share} = $share;
-    },
+    default => sub { Net::Curl::Share->new({ stamp => time }) },
     lazy    => 1,
-    weak_ref=> 1,
 );
 
-# WHYYYYYYY?????? WHYYYYYYYYYYY????????
-sub DEMOLISH {
-    my ($self) = @_;
-    my $share = $self->share // '';
-    delete $share{$share}
-        if exists $share{$share};
-}
-
 
 has stats       => (is => 'ro', isa => 'AnyEvent::Net::Curl::Queued::Stats', default => sub { AnyEvent::Net::Curl::Queued::Stats->new }, lazy => 1);
 
@@ -91,6 +79,7 @@ has unique      => (is => 'ro', isa => 'HashRef[Str]', default => sub { {} });
 
 has watchdog    => (is => 'ro', isa => 'Maybe[Ref]', writer => 'set_watchdog', clearer => 'clear_watchdog', predicate => 'has_watchdog');
 
+
 sub BUILD {
     my ($self) = @_;
 
@@ -106,6 +95,19 @@ sub BUILD {
     eval { $self->share->setopt(Net::Curl::Share::CURLSHOPT_SHARE, Net::Curl::Share::CURL_LOCK_DATA_SSL_SESSION) };
 }
 
+sub BUILDARGS {
+    my $class = shift;
+    if (@_ == 1 and q(HASH) eq ref $_[0]) {
+        return shift;
+    } elsif (@_ % 2 == 0) {
+        return { @_ };
+    } elsif (@_ == 1) {
+        return { max => shift };
+    } else {
+        confess 'Should be initialized as ' . $class . '->new(Hash|HashRef|Int)';
+    }
+}
+
 
 sub start {
     my ($self) = @_;
@@ -215,7 +217,7 @@ AnyEvent::Net::Curl::Queued - Any::Moose wrapper for queued downloads via Net::C
 
 =head1 VERSION
 
-version 0.035
+version 0.036
 
 =head1 SYNOPSIS
 
@@ -252,7 +254,7 @@ version 0.035
 
             for my $link (@links) {
                 $self->queue->prepend(sub {
-                    CrawlApache->new({ initial_url => $link });
+                    CrawlApache->new($link);
                 });
             }
         }
@@ -272,7 +274,7 @@ version 0.035
 
     my $q = AnyEvent::Net::Curl::Queued->new;
     $q->append(sub {
-        CrawlApache->new({ initial_url => 'http://localhost/manual/' })
+        CrawlApache->new('http://localhost/manual/')
     });
     $q->wait;
 
@@ -345,7 +347,11 @@ L<LWP::UserAgent>: no parallelism, no queueing. L<WWW::Mechanize> is built on to
 
 =item *
 
-L<LWP::Curl>: L<LWP::UserAgent>-alike interface for L<WWW::Curl>. No parallelism, no queueing. Fast and simple to use;
+L<LWP::Protocol::Net::Curl>: I<drop-in> replacement for L<LWP::UserAgent>, L<WWW::Mechanize> and their derivatives to use L<Net::Curl> as a backend;
+
+=item *
+
+L<LWP::Curl>: L<LWP::UserAgent>-alike interface for L<WWW::Curl>. Not a I<drop-in>, no parallelism, no queueing. Fast and simple to use;
 
 =item *
 
@@ -357,7 +363,7 @@ L<HTTP::Lite>: no parallelism, no queueing. Also fast;
 
 =item *
 
-L<Furl>: no parallelism, no queueing. B<Very> fast;
+L<Furl>: no parallelism, no queueing. B<Very> fast, despite being pure-Perl;
 
 =item *
 
@@ -377,14 +383,15 @@ L<Parallel::Downloader>: queued parallel downloads via L<AnyEvent::HTTP>. Very f
 
 (see also: L<CPAN modules for making HTTP requests|http://neilb.org/reviews/http-requesters.html>)
 
-Obviously, the bottleneck of any kind of download agent is the connection itself.
-However, socket handling and header parsing add a lots of overhead.
+Obviously, every download agent is (or, ideally, should be) I<I/O bound>.
+However, it is not uncommon for large concurrent batch downloads to hog the processor cycles B<before> consuming the full network bandwidth.
+The proposed benchmark measures the request rate of several concurrent download agents, trying hard to make all of them I<CPU bound> (by removing the I/O constraint).
+On practice, this benchmark results mean that download agents with lower request rate are less appropriate for parallelized batch downloads.
+On the other hand, download agents with higher request rate are more likely to reach the full capacity of a network link while still leaving spare resources for data parsing/filtering.
 
 The script F<eg/benchmark.pl> compares L<AnyEvent::Net::Curl::Queued> against several other download agents.
-Only L<AnyEvent::Net::Curl::Queued> itself, L<AnyEvent::Curl::Multi>, L<Parallel::Downloader>, L<Mojo::UserAgent> and L<lftp|http://lftp.yar.ru/> support parallel connections natively;
+Only L<AnyEvent::Net::Curl::Queued> itself, L<AnyEvent::Curl::Multi>, L<Parallel::Downloader>, L<Mojo::UserAgent> and L<lftp|http://lftp.yar.ru/> support concurrent downloads natively;
 thus, L<Parallel::ForkManager> is used to reproduce the same behaviour for the remaining agents.
-Both L<AnyEvent::Curl::Multi> and L<LWP::Curl> are frontends for L<WWW::Curl>.
-L<Parallel::Downloader> uses L<AnyEvent::HTTP> as it's backend.
 
 The download target is a copy of the L<Apache documentation|http://httpd.apache.org/docs/2.2/> on a local Apache server.
 The test platform configuration:
@@ -401,32 +408,32 @@ Ubuntu 11.10 (64-bit);
 
 =item *
 
-Perl v5.16.1 (installed via L<perlbrew>);
+Perl v5.16.2 (installed via L<perlbrew>);
 
 =item *
 
-libcurl 7.27.0 (without AsynchDNS, which slows down L<curl_easy_init()|http://curl.haxx.se/libcurl/c/curl_easy_init.html>).
+libcurl/7.28.0 (without AsynchDNS, which slows down L<curl_easy_init()|http://curl.haxx.se/libcurl/c/curl_easy_init.html>).
 
 =back
 
 The script F<eg/benchmark.pl> uses L<Benchmark::Forking> and L<Class::Load> to keep UA modules isolated and loaded only once.
 
-    $ perl benchmark.pl --count 100 --parallel 4 --repeat 5
+    $ perl benchmark.pl --count 100 --parallel 8 --repeat 10
 
-                             Request rate WWW::M LWP::UA Mojo::UA HTTP::Tiny HTTP::Lite AE::C::M P::D lftp YADA Furl wget curl LWP::Curl
-    WWW::Mechanize v1.72            303/s     --    -65%     -80%       -82%       -85%     -86% -91% -91% -93% -95% -96% -96%      -97%
-    LWP::UserAgent v6.04            873/s   187%      --     -44%       -48%       -58%     -60% -74% -74% -79% -87% -89% -89%      -90%
-    Mojo::UserAgent v3.39          1558/s   412%     78%       --        -7%       -24%     -29% -54% -54% -63% -76% -80% -80%      -82%
-    HTTP::Tiny v0.017              1672/s   451%     92%       8%         --       -19%     -24% -51% -51% -60% -74% -79% -79%      -81%
-    HTTP::Lite v2.4                2058/s   577%    136%      32%        23%         --      -6% -39% -39% -51% -68% -74% -74%      -77%
-    AnyEvent::Curl::Multi v1.1     2203/s   624%    152%      41%        31%         7%       -- -35% -35% -47% -66% -72% -72%      -75%
-    Parallel::Downloader v0.121560 3378/s  1015%    288%     118%       102%        65%      54%   --  -0% -19% -48% -57% -57%      -61%
-    lftp v4.3.1                    3401/s  1018%    289%     118%       103%        65%      55%   0%   -- -19% -48% -57% -57%      -61%
-    YADA v0.027                    4167/s  1276%    379%     169%       150%       103%      90%  23%  23%   -- -36% -47% -47%      -52%
-    Furl v0.40                     6502/s  2041%    645%     318%       288%       216%     196%  92%  91%  56%   -- -17% -18%      -26%
-    wget v1.12                     7874/s  2493%    803%     406%       371%       283%     258% 133% 132%  88%  21%   --  -0%      -10%
-    curl v7.27.0                   7899/s  2501%    806%     408%       372%       284%     260% 133% 133%  89%  22%   0%   --      -10%
-    LWP::Curl v0.12                8757/s  2780%    902%     462%       423%       326%     298% 158% 158% 109%  35%  11%  11%        --
+                             Request rate WWW::M LWP::UA Mojo::UA HTTP::Lite HTTP::Tiny AE::C::M lftp P::D YADA Furl curl wget LWP::Curl
+    WWW::Mechanize v1.72            231/s     --    -59%     -85%       -87%       -89%     -90% -93% -93% -94% -97% -98% -98%      -98%
+    LWP::UserAgent v6.04            567/s   145%      --     -64%       -68%       -72%     -77% -82% -83% -85% -92% -94% -95%      -96%
+    Mojo::UserAgent v3.54          1590/s   589%    181%       --       -10%       -22%     -34% -49% -53% -59% -76% -83% -87%      -88%
+    HTTP::Lite v2.4                1770/s   666%    213%      11%         --       -13%     -27% -44% -48% -54% -74% -81% -85%      -86%
+    HTTP::Tiny v0.024              2030/s   779%    259%      28%        15%         --     -16% -36% -40% -48% -70% -78% -83%      -84%
+    AnyEvent::Curl::Multi v1.1     2430/s   952%    329%      53%        37%        20%       -- -23% -29% -37% -64% -74% -80%      -81%
+    lftp v4.3.1                    3150/s  1262%    456%      98%        78%        55%      30%   --  -8% -19% -53% -66% -74%      -75%
+    Parallel::Downloader v0.121560 3410/s  1375%    502%     114%        92%        68%      40%   8%   -- -12% -49% -64% -72%      -73%
+    YADA v0.036                    3880/s  1579%    585%     144%       119%        91%      60%  23%  14%   -- -42% -59% -68%      -70%
+    Furl v1.00                     6700/s  2795%   1082%     320%       278%       229%     175% 113%  96%  72%   -- -29% -45%      -48%
+    curl v7.28.0                   9380/s  3953%   1554%     488%       429%       361%     285% 197% 175% 141%  40%   -- -23%      -27%
+    wget v1.12                    12100/s  5139%   2038%     661%       584%       496%     398% 285% 255% 212%  81%  29%   --       -5%
+    LWP::Curl v0.12               12800/s  5418%   2152%     701%       620%       528%     425% 305% 274% 229%  91%  36%   5%        --
 
     (output formatted to show module versions at row labels and keep column labels abbreviated)
 
@@ -539,6 +546,9 @@ For lazy initialization, wrap the worker in a C<sub { ... }>, the same way you d
 
 Process queue.
 
+=for Pod::Coverage BUILD
+BUILDARGS
+
 =head1 CAVEAT
 
 =over 4
